<?php
/*
Plugin Name: Zip Gateway for WooCommerce
Description: Use Zip as a payment gateway for WooCommerce.
Author: Zip Co Limited
Author URI: https://zip.co
Version: 1.4.0
WC requires at least: 3.1.0
WC tested up to: 5.5
*/

add_action( 'plugins_loaded', 'quadpay_wc_gateway', 0 );

function quadpay_wc_gateway() {

	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}

	class WC_Gateway_QuadPay extends WC_Payment_Gateway {

		/**
		 * @var $_instance WC_Gateway_QuadPay The reference to the singleton instance of this class
		 */
		private static $_instance = NULL;

		/**
		 * @var boolean Whether or not logging is enabled
		 */
		public static $log_enabled = false;

		/**
		 * @var WC_Logger Logger instance
		 */
		public static $log = false;
	/**
		 * @var string
		 */
		private $client_id;
		/**
		 * @var string
		 */
		private $client_secret;
		/**
		 * @var string
		 */
		private $testmode;
		/**
		 * @var string
		 */
		private $product_page_widget_customization;
		/**
		 * @var string
		 */
		private $product_page_widget_wrapper;
		/**
		 * @var string
		 */
		private $cart_widget_customization;
		/**
		 * @var string
		 */
		private $cart_widget_wrapper;
		/**
		 * @var string
		 */
		private $minimum_amount;
		/**
		 * @var string
		 */
		private $maximum_amount;
		/**
		 * @var array
		 */
		private $environments;
		/**
		 * @var string
		 */
		public $orderurl;
		/**
		 * @var string
		 */
		private $configurationUrl;
		/**
		 * @var string
		 */
		private $order_check_time;
		/**
		 * @var string
		 */
		private $ccIconsBase;
		/**
		 * @var string
		 */
		private $widget_backward_compatibility = false;

		/**
		 * Main WC_Gateway_QuadPay Instance
		 *
		 * Used for WP-Cron jobs when
		 *
		 * @since 1.0
		 * @return WC_Gateway_QuadPay Main instance
		 */
		public static function instance() {
			if (is_null(self::$_instance)) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		public function __construct() {

			$this->id = 'quadpay';

			$this->method_title = __( 'Zip - Pay in 4 installments', 'woo_quadpay' );
			$this->method_description = __( 'Use Zip as a credit or debit card processor for WooCommerce.', 'woo_quadpay' );

			$this->title = __( 'Zip - Pay in 4 installments', 'woo_quadpay');
			$this->description = __('Pay in 4 installments.','woo_quadpay');

			$this->supports = array( 'products', 'refunds' );

			$this->icon = plugin_dir_url( __FILE__ ) . 'assets/images/zip-logo-color.svg';
			$this->ccIconsBase = plugin_dir_url( __FILE__ ) . 'assets/images/cc/';

			self::get_options();

			// Load the form fields.
			$this->init_environment_config();

			// Load the form fields.
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();

			if ( false !== $this->testmode ) {
				$api_url = $this->environments[ $this->testmode ]['api_url'];
			} else {
				$api_url = '';
			}

			$this->orderurl         = $api_url . '/order';
			$this->configurationUrl = $api_url . '/configuration';

			self::$log_enabled = ($this->get_option( 'logging' ) === 'yes');

			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			add_action( 'woocommerce_thankyou', array( $this, 'payment_callback' ) );

			// Don't enable QuadPay if the amount limits are not met
			add_filter( 'woocommerce_available_payment_gateways', array( $this, 'check_cart_within_limits' ), 99, 1 );

			add_action( 'woocommerce_update_options_checkout', array( $this, 'save_options_page') );

		}

		/**
		 * Collect options
		 *
		 * @since 1.3.7
		 */
		private function get_options() {

			$this->client_id                         = $this->get_option( 'client_id' );
			$this->client_secret                     = $this->get_option( 'client_secret' );
			$this->testmode                          = $this->get_option( 'testmode' );
			$this->product_page_widget_customization = $this->get_option( 'product_page_widget_customization' );
			$this->product_page_widget_wrapper       = $this->get_option( 'product_page_widget_wrapper' );
			$this->cart_widget_customization         = $this->get_option( 'cart_widget_customization' );
			$this->cart_widget_wrapper               = $this->get_option( 'cart_widget_wrapper' );
			$this->minimum_amount                    = $this->get_option( 'quadpay-amount-minimum' );
			$this->maximum_amount                    = $this->get_option( 'quadpay-maximum-amount' );
			// Default value 24h
			$this->order_check_time                =
				( !empty( $this->get_option( 'order_check_time' ) ) ? $this->get_option( 'order_check_time' ) : 'DAY_IN_SECONDS');

			$this->widget_backward_compatibility = $this->get_option( 'widget_backward_compatibility', false );

		}

		/**
		 * Check if this gateway is enabled
		 */
		public function is_available() {

			if ( 'yes' == $this->enabled ) {

				if ( ! ( $this->client_id && $this->client_secret ) ) {

					return false;

				}

				return true;

			}

			return false;

		}

		/**
		 * Initialise Gateway Settings Form Fields
		 *
		 * @since 1.0.0
		 */
		public function init_form_fields() {

			$env_values = array();

			foreach ( $this->environments as $key => $item ) {
				$env_values[ $key ] = $item["name"];
			}

			$this->form_fields = array(
				'api_title      ' => array(
					'title'       => __( 'API Settings', 'woo_quadpay ' ),
					'type'        => 'title',
					'description' => __( '' ),
				),
				'enabled'     => array(
					'title'   => __( 'Enable/Disable', 'woo_quadpay' ),
					'type'    => 'checkbox',
					'label'   => __( 'Enable Zip', 'woo_quadpay' ),
					'default' => 'yes',
				),
				'testmode'                  => array(
					'title'       => __( 'Test mode', 'woo_quadpay' ),
					'label'       => __( 'Enable Test mode', 'woo_quadpay' ),
					'type'        => 'select',
					'options'     => $env_values,
					'description' => __( 'When in Test/Sandbox mode, no transactions will actually be processed.', 'woo_quadpay' ),
				),
				'client_id'       => array(
					'title'       => __( 'Client ID', 'woo_quadpay' ),
					'type'        => 'text',
					'description' => __( 'Zip Client ID credential', 'woo_quadpay' ),
					'default'     => __( '', 'woo_quadpay' ),
				),
				'client_secret'   => array(
					'title'       => __( 'Client Secret', 'woo_quadpay' ),
					'type'        => 'text',
					'description' => __( 'Zip Client Secret credential', 'woo_quadpay' ),
					'default'     => __( '', 'woo_quadpay' ),
				),
				'logging'         => array(
					'title'       => __( 'Logging', 'woo_quadpay' ),
					'label'       => __( 'Log debug messages', 'woo_quadpay' ),
					'type'        => 'checkbox',
					'description' => __( 'Save debug messages to the WooCommerce System Status log.', 'woo_quadpay' ),
					'default'     => 'no',
					'desc_tip'    => true,
				),
				'order_check_settings' => array(
					'title'       => __( 'Order Check', 'woo_quadpay ' ),
					'type'        => 'title',
					'description' => __( '' ),
				),
				'order_check_time'                  => array(
					'title'       => __( 'Order Check Timeframe', 'woo_quadpay' ),
					'label'       => __( 'Order Check Timeframe', 'woo_quadpay' ),
					'type'        => 'select',
					'options'     => array(
							'DAY_IN_SECONDS'   => __( '24h hours', 'woo_quadpay' ),
							'WEEK_IN_SECONDS'  => __( 'One Week', 'woo_quadpay' ),
							'MONTH_IN_SECONDS' => __( 'One Month', 'woo_quadpay' ),
							'YEAR_IN_SECONDS'  => __( 'One Year', 'woo_quadpay' ),
					),
					'description' => __( 'How far back should authorized, not captured orders be checked for status change.', 'woo_quadpay' ),
				),
				'widget_settings' => array(
					'title'       => __( 'Widget Settings', 'woo_quadpay ' ),
					'type'        => 'title',
					'description' => __( '' ),
				),
				'enable_product_widget'     => array(
					'title'   => __( 'Product Page Widget', 'woo_quadpay' ),
					'type'    => 'checkbox',
					'label'   => __( 'Enable Product Page Widget', 'woo_quadpay' ),
					'default' => 'yes',
				),
				'enable_cart_widget'        => array(
					'title'   => __( 'Cart Page Widget', 'woo_quadpay' ),
					'type'    => 'checkbox',
					'label'   => __( 'Enable Cart Page Widget', 'woo_quadpay' ),
					'default' => 'yes',
				),
				'merchant_widget_reference' => array(
					'title'   => __( 'Widget Reference', 'woo_quadpay' ),
					'type'    => 'text',
					'default' => __( '', 'woo_quadpay' ),
				),
				'widget_customization'      => array(
					'title'       => __( 'Widget Customization', 'woo_quadpay ' ),
					'type'        => 'title',
					'description' => __( 'Customize the look of the Zip widget.', 'woo_quadpay' ),
				),
				'product_page_widget_customization' => array(
					'title'             => __( 'Product Page Widget Customization', 'woo_quadpay' ),
					'type'              => 'textarea',
					'default'           => __( '', 'woo_quadpay' ),
					'description'       => sprintf( __( 'Optional: To customize the appearance of the widget, you can pass attributes. Each attribute should be added on a new line. For further technical support, please refer to the <a href="%s" target="_blank">documentation</a>.', 'woo_quadpay' ), 'https://docs.quadpay.com/docs/widget-integration-v3' ),
				),
				'product_page_widget_wrapper'       => array(
					'title'             => __( 'Product Page Widget Wrapper', 'woo_quadpay' ),
					'type'              => 'textarea',
					'default'           => __( 'margin: 0 0 10px 0;', 'woo_quadpay' ),
					'description'       => sprintf( __( 'Optional: To adjust the spacing around the widget, you can pass CSS styles. Please refer to the <a href="%s" target="_blank">documentation</a>. A technical background may be helpful when passing these styles.', 'woo_quadpay' ), 'https://docs.quadpay.com/docs/widget-integration-v3' ),
				),
				'cart_widget_customization'         => array(
					'title'             => __( 'Cart Widget Customization', 'woo_quadpay' ),
					'type'              => 'textarea',
					'default'           => __( '', 'woo_quadpay' ),
					'description'       => sprintf( __( 'Optional: To customize the appearance of the widget, you can pass attributes. Each attribute should be added on a new line. For further technical support, please refer to the <a href="%s" target="_blank">documentation</a>.', 'woo_quadpay' ), 'https://docs.quadpay.com/docs/widget-integration-v3' ),
				),
				'cart_widget_wrapper'               => array(
					'title'             => __( 'Cart Widget Wrapper', 'woo_quadpay' ),
					'type'              => 'textarea',
					'default'           => __( 'margin: 0 0 10px 0;', 'woo_quadpay' ),
					'description'       => sprintf( __( 'Optional: To adjust the spacing around the widget, you can pass CSS styles. Please refer to the <a href="%s" target="_blank">documentation</a>. A technical background may be helpful when passing these styles.', 'woo_quadpay' ), 'https://docs.quadpay.com/docs/widget-integration-v3' ),
				),
				'widget_backward_compatibility'        => array(
					'title'   => __( 'Old Widget Compatibility', 'woo_quadpay' ),
					'type'    => 'checkbox',
					'label'   => __( 'Enable backward compatibility with an old widget', 'woo_quadpay' ),
					'description'   => __( 'Use this only if you have custom implementation of old &lt;quadpay-widget&gt; tag on your site.', 'woo_quadpay' ),
					'default' => 'no',
				),
			);

		} // End init_form_fields()

		/**
		 * Init Environment Options
		 *
		 * @since 1.2.3
		 */
		public function init_environment_config() {
			if ( empty( $this->environments ) ) {
				//config separated for ease of editing
				require( 'config/config.php' );
				/** @var array $environments */
                $this->environments = $environments;
			}
		}

		/**
		 * Admin Panel Options
		 *
		 * @since 1.0.0
		 */
		public function admin_options() {

			wp_cache_flush();

			$settings = get_option( 'woocommerce_quadpay_settings' );
			$min_amount = $settings['quadpay-amount-minimum'];
			$max_amount = $settings['quadpay-maximum-amount'];
			unset( $settings );

			?>
			<h2><?php _e( 'Zip Gateway', 'woo_quadpay' ); ?></h2>
			<?php
			$login = (bool) ( false === $this->get_quadpay_authorization_code( 'login check' ) ? false : true );
			?>
			<p style="color: <?php echo $login ? 'green' : 'red' ?>">
				<?php echo( !$login ? __( 'Zip login failed. Please check your Client ID and Client Secret.', 'woo_quadpay' ) : __( 'Validation of Zip login credentials passed successful.', 'woo_quadpay' ) ); ?>
			</p>

			<h3 class="wc-settings-sub-title " id="woocommerce_quadpay_payment_limits"><?php _e( 'Payment Limits', 'woo_quadpay ' )?></h3>
			<p><?php _e( 'The possible minimum and maximum payment limit for the Zip payment.', 'woo_quadpay' )?></p>
			<table class="form-table">
				<tbody><tr valign="top">
					<th scope="row" class="titledesc">
						<label for="woocommerce_quadpay_min_amount"><?php _e( 'Minimum Amount', 'woo_quadpay' ) ?></label>
					</th>
					<td class="forminp">
						<?php echo $min_amount ?>
					</td>
				</tr>
				<tr valign="top">
					<th scope="row" class="titledesc">
						<label for="woocommerce_quadpay_max_amount"><?php _e( 'Maximum Amount', 'woo_quadpay' ) ?></label>
					</th>
					<td class="forminp">
						<?php echo $max_amount ?>
					</td>
				</tr>
				</tbody>
			</table>

			<table class="form-table">
				<?php
				// Generate the HTML For the settings form.
				$this->generate_settings_html( $this->form_fields, true);
				?>
			</table><!--/.form-table-->
			<?php
		} // End admin_options()

		/**
		 * Display payment options on the checkout page
		 *
		 * @since 1.0.0
		 */
		public function payment_fields() {

			// Payment form
			if ( $this->testmode != 'production' ) : ?>
				<p><?php _e( 'TEST MODE ENABLED', 'woo_quadpay' ); ?></p>

			<?php endif;

			if ( $this->description ) {
				echo '<p>' . $this->description . '</p>';
			}

			$ccTypes = [
				'visa',
				'mastercard',
				'amex',
				'discover'
			];

			echo '<p><span>' . __('Credit and debit cards accepted:', 'woo_quadpay') . '</span>';
			foreach ($ccTypes as $ccType) {
				echo '<img class="qp_cc_type" src="' . $this->ccIconsBase . $ccType . '.svg" style="height:23px; margin-left: 5px;">';
			}
			echo '</p>';
		}

		/**
		 * Request an order token from QuadPay
		 *
		 * @param string $reference where does the request come from
		 *
		 * @return  string or boolean false if no token generated
		 * @since 1.0.0
		 */
		public function get_quadpay_authorization_code( $reference = 'get_quadpay_authorization_code' ) {

			$access_token = wp_cache_get( 'quadpay_access_token', 'quadpay' );

			if ( false !== $access_token ) {
				$this->log( 'Access token from cache' );
				return $access_token;
			}

			if ( false === $this->apiKeysAvailable( $reference ) ) {
				return false;
			}

			$AuthURL = $this->environments[ $this->testmode ]['auth_url'];

			$AuthBody = array(
				'client_id'     => $this->client_id,
				'client_secret' => $this->client_secret,
				'audience'      => $this->environments[ $this->testmode ]['auth_audience'],
				'grant_type'    => 'client_credentials',
			);

			$args = array(
				'method' => 'POST',
				'headers' => array( 'Content-Type' => 'application/json'),
				'body' =>  json_encode( $AuthBody )
			);

			$response = wp_remote_post($AuthURL ,$args);

			if ( is_wp_error( $response ) ) {
				$this->log( '[Authentication] QuadPay API Error: ' . $response->get_error_message() );
				return false;
			}

			$body = json_decode(wp_remote_retrieve_body($response));

			$response_code = wp_remote_retrieve_response_code( $response );

			if ( $response_code == 200 ) {

				$this->log( '[Retrieve response] Response code ' . $response_code . ' valid');

				//store this in cache
				wp_cache_set('quadpay_access_token', $body->access_token, 'quadpay', $body->expires_in);

				$this->log( '[Retrieve response] Access token available.' );

				return $body->access_token;

			} else {

				if ( 401 == $response_code ) {

					$this->log( '[Retrieve response] QuadPay login failed. Please check login credentials.' );

				}
				else {

					$this->log( '[Retrieve response] Error response code ' . $response_code );

				}

				return false;
			}
		}

		private function apiKeysAvailable( $reference ) {

			if ( empty( $this->client_id ) || empty( $this->client_secret ) ) {
				$this->log( '[API key availability check] API keys not available. Reference: ' . $reference );

				return false;
			}

			$this->log( '[API key availability check] API keys available. Reference: ' . $reference );
			//$this->log( '[API key availability check] Client ID: ' . $this->client_id );
			//$this->log( '[API key availability check] Client secret: ' . $this->client_secret );

			return true;
		}

		public function save_options_page() {

			$this->log( '[Saved settings] start' );
			wp_cache_flush();
			self::get_options();
			$this->update_payment_limits();
			$this->log( '[Saved settings] end' );

		}

		public function update_payment_limits(){

			// Get existing limits
			$settings = get_option('woocommerce_quadpay_settings');

			if( false === $this->apiKeysAvailable( 'update_payment_limits' ) ){
				return false;
			}

			$this->log( '[Updating payment limits] request values');

			$headers = array( 'headers' => array(
					'Authorization' => 'Bearer ' . $this->get_quadpay_authorization_code( 'update_payment_limits' ),
				),
			);

			$response      = wp_remote_get( $this->configurationUrl, $headers );
			$body          = json_decode( wp_remote_retrieve_body( $response ) );
			$response_code = wp_remote_retrieve_response_code( $response );

			$this->log( '[Updating payment limits] response code: ' . $response_code );
			$this->log( '[Updating payment limits] response: ' . print_r( $body, true ) );

			if ( $response_code == 200 ) {

				$settings['quadpay-amount-minimum'] = $body->minimumAmount;
				$settings['quadpay-maximum-amount'] = $body->maximumAmount;

				update_option( 'woocommerce_quadpay_settings', $settings );

				return true;

			}

			return false;

		}

		/**
		 * Process the payment and return the result
		 * - redirects the customer to the pay page
		 *
		 * @param int $order_id
		 *
		 * @since 1.0.0
		 * @return array
		 */
		public function process_payment( $order_id ) {

			if ( function_exists( 'wc_get_order' ) ) {
				$order = wc_get_order( $order_id );
			} else {
				$order = new WC_Order( $order_id );
			}

			// Get the authorization token
			$access_token = $this->get_quadpay_authorization_code( 'process_payment');

			//Process here
			$orderitems = $order->get_items();
			$items = array();

			if ( count($orderitems ) ) {

				foreach ( $orderitems as $item ) {

					if ( $item['variation_id'] ) {

						if ( function_exists( "wc_get_product" ) ) {
							$product = wc_get_product( $item['variation_id'] );
						} else {
							$product = new WC_Product( $item['variation_id'] );
						}

					} else {

						if ( function_exists( "wc_get_product" ) ) {
							$product = wc_get_product( $item['product_id'] );
						} else {
							$product = new WC_Product( $item['product_id'] );
						}

					}

					$items[] = array(
						'name'     => $item['name'],
						'sku'      => $product->get_sku(),
						'quantity' => $item['qty'],
						'price'    => number_format( ( $item['line_subtotal'] / $item['qty'] ), 2, '.', '' ),
					);

				}
			}

			//calculate total shipping amount
			if ( method_exists( $order, 'get_shipping_total') ){
				//WC 3.0
				$shipping_total = $order->get_shipping_total();
			} else {
				//WC 2.6.x
				$shipping_total = $order->get_total_shipping();
			}

			$body = array(
				'amount'            => number_format( $order->get_total(), 2, '.', '' ),
				'consumer'          => array(
					'phoneNumber' => $this->get_order_prop( $order, 'billing_phone' ),
					'givenNames'  => $this->get_order_prop( $order, 'billing_first_name' ),
					'surname'     => $this->get_order_prop( $order, 'billing_last_name' ),
					'email'       => $this->get_order_prop( $order, 'billing_email' ),
				),
				'billing'           => array(
					'addressLine1' => $this->get_order_prop( $order, 'billing_address_1' ),
					'addressLine2' => $this->get_order_prop( $order, 'billing_address_2' ),
					'city'         => $this->get_order_prop( $order, 'billing_city' ),
					'state'        => $this->get_order_prop( $order, 'billing_state' ),
					'postcode'     => $this->get_order_prop( $order, 'billing_postcode' ),
				),
				'shipping'          => array(
					'addressLine1' => $this->get_order_prop( $order, 'shipping_address_1' ),
					'addressLine2' => $this->get_order_prop( $order, 'shipping_address_2' ),
					'city'         => $this->get_order_prop( $order, 'shipping_city' ),
					'state'        => $this->get_order_prop( $order, 'shipping_state' ),
					'postcode'     => $this->get_order_prop( $order, 'shipping_postcode' ),
				),
				'description'       => 'string',
				'items'             => $items,
				'merchant'          => array(
					'redirectConfirmUrl' => $this->get_return_url( $order ) . '&status=confirmed',
					'redirectCancelUrl'  => $this->get_return_url( $order ) . '&status=cancelled',
				),
				'merchantReference' => $order_id,
				'taxAmount'         => $order->get_total_tax(),
				'shippingAmount'    => $shipping_total,
			);

			$headers = array(
				'Content-Type'  => 'application/json',
				'Authorization' => 'Bearer ' . $access_token,
			);

			$order_args = array(
				'method'  => 'POST',
				'headers' => $headers,
				'body'    => json_encode( $body ),
				'timeout' => 30,
			);

			$this->log( 'POST Order request: '.print_r( $order_args,true ) );

			$APIURL = $this->orderurl;

			$order_response = wp_remote_post( $APIURL, $order_args );

			$response_code = wp_remote_retrieve_response_code( $order_response );

			$order_body = json_decode( wp_remote_retrieve_body( $order_response ) );

			$this->log( 'POST Order response: ' . print_r( $order_body,true ) );

			$valid_response_code = array( 200, 201 );

			if ( ! in_array( $response_code, $valid_response_code ) ) {
				// Couldn't generate token
				$order->add_order_note(__('Unable to generate the order token. Payment couldn\'t proceed.', 'woo_quadpay' ) );

				wc_add_notice(__('Sorry, there was a problem preparing your payment. Please try again.', 'woo_quadpay'),'error');

				return;

			} else {

				// Order token successful, save it so we can confirm it later
				update_post_meta( $order_id,'_quadpay_order_token', $order_body->token );

				$redirect = $order_body->redirectUrl;

				return array(
					'result' 	=> 'success',
					'redirect'	=> $redirect
				);

			}

		}

		public function get_quadpay_order_id( $order_id ) {

			$quadpay_order_id = get_post_meta( $order_id, '_quadpay_order_id', true );

			if ( ! empty( $quadpay_order_id ) ) {
				return $quadpay_order_id;
			}

			$access_token = $this->get_quadpay_authorization_code( 'get_quadpay_order_id');
			$order_token  = get_post_meta( $order_id, '_quadpay_order_token', true );

			if ( empty( $order_token ) ) {
				return false;
			}

			//check if quadpay order id already exists

			//get the order id and save in postmeta
			$get_args = array(
				'headers' 	=> array(
					'Content-Type' => 'application/json',
					'Authorization' => 'Bearer '. $access_token
				),
			);

			$getOrderUrl = $this->orderurl . '?token=' . $order_token;

			$get_response = wp_remote_get( $getOrderUrl, $get_args );
			$get_body     = json_decode( wp_remote_retrieve_body( $get_response ) );

			$this->log( 'Get Order result: ' . print_r( $get_body, true ) );

			$quadpay_order_id     = $get_body->orderId;
			$quadpay_order_status = $get_body->orderStatus;

			//store order id for future purposes
			if ( $quadpay_order_status != "Approved" ) {
				return false;
			}

			update_post_meta($order_id,'_quadpay_order_id', $quadpay_order_id);
			update_post_meta($order_id,'_quadpay_order_status', $quadpay_order_status);

			return $quadpay_order_id;

		}

		/**
		 * @param $order_id
		 */
		public function payment_callback($order_id) {

			if ( function_exists( "wc_get_order" ) ) {
				$order = wc_get_order( $order_id );
			} else {
				$order = new WC_Order( $order_id );
			}

			if ( 'quadpay' !== $order->get_payment_method() ) {
				return;
			}

			//save the order id
			$quadpay_order_id = $this->get_quadpay_order_id($order_id);

			$order_token = get_post_meta($order_id, '_quadpay_order_token', true);
			if($quadpay_order_id == false){
				$order->add_order_note(sprintf(__('Failure to process the Zip payment.','woo_quadpay') ));
			}else if ( !empty($order_token) ) {

				$quadpay_order_id = $this->get_quadpay_order_id($order_id);

				//Update status for order
				if (isset($_GET['status'])) {

					$query_string = $_GET['status'];

					if ($query_string != NULL) {
						if ($query_string == 'confirmed') {

							$order->add_order_note(sprintf(__('Payment approved. Zip Order ID: '.$quadpay_order_id.' ','woo_quadpay') ));
							$order->payment_complete($quadpay_order_id);
							wc_empty_cart();

						}elseif ($query_string == 'cancelled') {

							$order->add_order_note(sprintf(__('Zip payment is pending approval. Zip Order ID: '.$quadpay_order_id.' ','woo_quadpay') ));
							$order->update_status( 'cancelled' );


						} elseif ($query_string == 'failure') {

							$order->add_order_note(sprintf(__('Zip payment declined. Order ID from Zip: '.$quadpay_order_id.' ','woo_quadpay') ));
							$order->update_status('failed');
						}

					}else{
						$order->update_status('pending');
					}
				}

			}

		}

		/**
		 * Check whether the cart amount is within payment limits
		 *
		 * @param  array $gateways Enabled gateways
		 * @return  array Enabled gateways, possibly with QuadPay removed
		 * @since 1.0.0
		 */
		public function check_cart_within_limits($gateways) {

			if ( is_admin() ) {
				return $gateways;
			}

			$total = WC()->cart->total;

			$minAmount = isset( $this->minimum_amount ) ? $this->minimum_amount : NULL;
			$maxAmount = isset( $this->maximum_amount ) ? $this->maximum_amount : NULL;

			if ( $minAmount == NULL || $maxAmount == NULL ) {

				$this->log("Requesting payment limit update");

				$status = $this->update_payment_limits();

				if ( $status ) {

					$settings = get_option( 'woocommerce_quadpay_settings' );

					$minAmount = $settings['quadpay-amount-minimum'];
					$maxAmount = $settings['quadpay-maximum-amount'];

				}

			} else {

				$this->log("Using cached payment limits");

			}

			$pbi = ( $total >= $minAmount && $total <= $maxAmount );

			if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
				global $woocommerce;
				$billing_country = $woocommerce->customer->country;
			} else {
				 // make sure to call the member function only if not null
				 // @since 1.3.11
				if ( null !== WC()->customer && null !== WC()->customer->get_billing_country() )
					$billing_country = WC()->customer->get_billing_country();
			}

			if ( ! $pbi || ( null != $billing_country || empty( $billing_country ) ) && 'US' != $billing_country  && 'CA' != $billing_country ) {
				unset( $gateways['quadpay'] );
			}

			return $gateways;

		}

		/**
		 * Can the order be refunded?
		 *
		 * @param  object $order WC_Order
		 * @return bool
		 */
		public function can_refund_order( $order ) {
			 return $order && $order->get_transaction_id();
		}

		/**
		 * Process a refund if supported
		 *
		 * @param object $order_id WC_Order
		 * @param float $amount
		 * @param string $reason
		 *
		 * @return  boolean True or false based on success
		 */
		public function process_refund( $order_id, $amount = null, $reason = '' ) {

			$quadpay_order_id = $this->get_quadpay_order_id( $order_id );

			$order = new WC_Order( $order_id );

			if ( empty( $quadpay_order_id ) ) {

				$order->add_order_note( sprintf( __( 'There was an error submitting the refund to Zip.', 'woo_quadpay' ) ) );

				return false;
			}

			$access_token  = $this->get_quadpay_authorization_code( 'process_refund' );
			$random_string = wp_generate_password( 8, false, false );

			$refund_args = array(
				'headers' => array(
					'Content-Type'  => 'application/json',
					'Authorization' => 'Bearer ' . $access_token,
				),
				'body'    => json_encode( array(
					'requestId'               => 'Order #' . $order_id . '-' . $random_string,
					'amount'                  => $amount,
					'merchantRefundReference' => 'Order #' . $order_id . '-' . $random_string,
				) ),
			);

			$refundOrderUrl = $this->orderurl.'/'.$quadpay_order_id.'/refund';

			$refund_response = wp_remote_post( $refundOrderUrl, $refund_args );
			$refund_body     = json_decode( wp_remote_retrieve_body( $refund_response ) );

			$this->log( 'Refund body: ' . print_r( $refund_body, true ) );

			$responsecode = intval( $refund_response['response']['code'] );

			if ( $responsecode == 201 || $responsecode == 200 ) {

				$order->add_order_note( sprintf( __( 'Refund of $%s successfully sent to Zip.', 'woo_quadpay' ), $amount ) );

				return true;

			} else {

				if ( $responsecode == 404 ) {
					$order->add_order_note( sprintf( __( 'Order not found on Zip.', 'woo_quadpay' ) ) );
				} else {
					$order->add_order_note( sprintf( __( 'There was an error submitting the refund to Zip.', 'woo_quadpay' ) ) );
				}

				return false;
			}

		}

		/**
		 * Logging method
		 * @param  string $message
		 */
		public static function log( $message ) {

			if ( self::$log_enabled ) {
				if ( empty( self::$log ) ) {
					self::$log = new WC_Logger();
				}
				self::$log->add( 'quadpay', $message );
			}

			if( WP_DEBUG && WP_DEBUG_LOG )
				error_log( $message );

		}

		/**
		 * Check the order status of all pending orders that didn't return to the thank you page or marked as Pending by QuadPay
		 *
		 * @since 1.3.0
		 */
		public function check_pending_orders() {

			// Get PENDING orders that may have been abandoned, or browser window closed after approved
			$args = array(
				'status'         => 'pending',
				'type'           => 'shop_order',
				'date_created'   => '>' . ( time() - DAY_IN_SECONDS ),
				'limit'          => -1,
				'payment_method' => 'quadpay',
			);
			$pending_orders = wc_get_orders( $args );

			if ( empty( $pending_orders ) ) {
				return;
			}

			foreach ( $pending_orders as $order ) {

				$order_id = $order->get_id();

				$quadpay_order_token = get_post_meta( $order_id, '_quadpay_order_token', true );

				// Check if there's a stored order token. If not, it's not an QuadPay order.
				if ( ! $quadpay_order_token ) continue;

				$this->log( 'Checking abandoned order for WC Order ID ' . $order_id . ', Zip Token ' . $quadpay_order_token );

				$response = wp_remote_get(
					$this->orderurl.'?token=' . $quadpay_order_token,
					array(
						'headers'=>array(
							'Authorization' => 'Bearer ' . $this->get_quadpay_authorization_code( 'check_pending_orders'),
						)
					)
				);

				$body = json_decode( wp_remote_retrieve_body( $response ) );

				$this->log( 'Checking abandoned order result: ' . print_r( $body,true ) );

				$response_code = wp_remote_retrieve_response_code( $response );

				// Check status of order
				if ( $response_code == 200 ) {

					// Check status of order
					if ( $body->orderStatus == "Approved" ) {

						$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment approved. Zip Order ID: %s', 'woo_quadpay' ), $body->orderId ) );
						$order->payment_complete( $order_id );

					} elseif ( $body->orderStatus == "Created" ) {

						$order->add_order_note( __( 'Checked payment status with Zip. Still pending approval.', 'woo_quadpay' ) );
						$order->update_status( 'pending' );

					} elseif ( $body->orderStatus == 'Abandoned' ) {

						$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment %s. Zip Order ID: %s', 'woo_quadpay' ), strtolower( $body->orderStatus ), $body->orderId ) );
						$order->update_status( 'cancelled' );

					} else {

						$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment %s. Zip Order ID: %s', 'woo_quadpay' ), strtolower( $body->orderStatus ), $body->orderId ) );
						$order->update_status( 'failed' );

					}

				} else {

					$order_date = method_exists( $order, 'get_date_created' ) ? $order->get_date_created() : $order->order_date;

					if ( strtotime( 'now' ) - strtotime( $order_date ) > 3600 ) {

						$order->add_order_note( sprintf( __( 'Pending Order Expired' ) ) );
						$order->update_status( 'cancelled' );

					}

				}

			}

		}

		/**
		 * Check the order status of all on-hold orders that didn't return to the thank you page or marked as Pending by QuadPay
		 *
		 * @since 1.3.0
		 */
		public function check_on_hold_orders() {

			$this->log( 'Start checking On Hold orders' );

			// Get ON-HOLD orders that are "pending" at QuadPay that need to be checked whether approved or denied
			$args = array(
				'status'         => 'on-hold',
				'type'           => 'shop_order',
				'date_created'   => '>' . ( time() - constant( $this->order_check_time ) ),
				'limit'          => -1,
				'payment_method' => 'quadpay',
			);

			$on_hold_orders = wc_get_orders( $args );

			if ( empty( $on_hold_orders ) ) {

				$this->log( 'No On Hold orders found (' . $this->order_check_time . ')' );
				$this->log( 'End checking On Hold orders' );

				return;
			}

			$this->log( count( $on_hold_orders ) . ' On Hold orders found' );

			foreach ( $on_hold_orders as $order ) {

				$order_id = $order->get_id();

				$quadpay_order_token = get_post_meta( $order_id, '_quadpay_order_token', true );

				// Check if there's a stored order token. If not, it's not an QuadPay order.
				if ( ! $quadpay_order_token ) continue;

				$this->log( 'Checking abandoned order for WC Order ID ' . $order_id . ', QuadPay Token ' . $quadpay_order_token );

				$response = wp_remote_get(
					$this->orderurl.'?token=' . $quadpay_order_token,
					array(
						'headers'=>array(
							'Authorization' => 'Bearer ' . $this->get_quadpay_authorization_code( 'check_on_hold_orders' ),
						)
					)
				);

				$body = json_decode( wp_remote_retrieve_body( $response ) );

				$this->log( 'Checking abandoned order result: ' . print_r( $body, true ) );

				$response_code = wp_remote_retrieve_response_code( $response );

				// Check status of order
				if ( $response_code == 200 ) {

					// Check status of order
					if ( $body->orderStatus == "Approved" ) {

						$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment approved. Zip Order ID: %s', 'woo_quadpay' ), $body->orderId ) );
						$order->payment_complete( $order_id );

					} elseif ( $body->orderStatus == "Created" ) {

						$order->add_order_note( __( 'Checked payment status with Zip. Still pending approval.', 'woo_quadpay' ) );
						$order->update_status( 'pending' );

					} elseif ( $body->orderStatus == 'Abandoned' ) {

						$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment %s. Zip Order ID: %s', 'woo_quadpay' ), strtolower( $body->orderStatus ), $body->orderId ) );
						$order->update_status( 'cancelled' );

					} else {

						$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment %s. Zip Order ID: %s', 'woo_quadpay' ), strtolower( $body->orderStatus ), $body->orderId ) );
						$order->update_status( 'failed' );

					}

				} else {

					$order_date = method_exists( $order, 'get_date_created' ) ? $order->get_date_created() : $order->order_date;

					if ( strtotime( 'now' ) - strtotime( $order_date ) > 3600 ) {

						$order->add_order_note( sprintf( __( 'On Hold Order Expired at Zip' ) ) );
						$order->update_status( 'cancelled' );

					}

				}

			}

		}

		/**
		 * Gets an order property.
		 *
		 * This method exists for WC 3.0+ compatibility.
		 */
		private function get_order_prop( WC_Order $order, $prop ) {

			$wc_version = defined( 'WC_VERSION' ) && WC_VERSION ? WC_VERSION : null;

			if ( $wc_version && version_compare( $wc_version, '3.0', '>=' ) && is_callable( array( $order, "get_{$prop}" ) ) ) {
				$value = $order->{"get_{$prop}"}();
			} else {
				$value = $order->$prop;
			}

			return $value;
		}

	}

	/**
	 * Add the QuadPay gateway to WooCommerce
	 *
	 * @param  array $methods Array of Payment Gateways
	 * @return  array Array of Payment Gateways
	 * @since 1.0.0
	 **/
	function add_quadpay_gateway( $methods ) {
		$methods[] = 'WC_Gateway_QuadPay';
		return $methods;
	}
	add_filter('woocommerce_payment_gateways', 'add_quadpay_gateway' );

	/**
	 * Check for the CANCELLED payment status
	 * We have to do this before the gateway initialises because WC clears the cart before initialising the gateway
	 *
	 * @since 1.0.0
	 */
	function quadpay_check_for_cancelled_payment() {
		// Check if the payment was cancelled
		if ( isset( $_GET['status'] ) && $_GET['status'] == "cancelled" && isset( $_GET['key'] ) && isset( $_GET['token'] ) ) {

			$gateway = WC_Gateway_QuadPay::instance();

			$order_id = wc_get_order_id_by_order_key( $_GET['key'] );

			if ( function_exists( "wc_get_order" ) ) {
				$order = wc_get_order( $order_id );
			} else {
				$order = new WC_Order( $order_id );
			}

			if ( $order ) {
				$gateway->log( 'Order ' . $order_id . ' payment cancelled by the customer while on the Zip checkout pages.' );
				$order->add_order_note( __( 'Payment cancelled by the customer while on the Zip checkout pages.', 'woo_quadpay' ) );

				if ( method_exists( $order, "get_cancel_order_url_raw" ) ) {
					wp_redirect( $order->get_cancel_order_url_raw() );
				} else {
					wp_redirect( $order->get_cancel_order_url() );
				}
				exit;
			}
		}
	}
	add_action( 'template_redirect', 'quadpay_check_for_cancelled_payment' );

	/**
	 * Showing the QuadPay widget on a simple product page
	 *
	 * @param $price_html
	 * @param $product
	 *
	 * @return string
	 */
	function quadpay_product_price_html_widget( $price_html, $product ) {

		if ( ! is_product() ) {
			return $price_html;
		}

		$settings = get_option( 'woocommerce_quadpay_settings' );

		if ( ! isset( $settings['enable_product_widget'] ) || $settings['enable_product_widget'] !== 'yes' || $settings['enabled'] !== 'yes' ) {
			return $price_html;
		}

		if ( 'simple' != $product->get_type() ) {
			return $price_html;
		}

		if ( did_action( 'woocommerce_before_add_to_cart_quantity' ) ) {
			return $price_html;
		}

		$widget_wrapper       = ! empty( $settings['product_page_widget_wrapper'] ) ? $settings['product_page_widget_wrapper'] : '';
		$widget_customization = ! empty( $settings['product_page_widget_customization'] ) ? $settings['product_page_widget_customization'] : '';

		$price = $product->get_price();
		$tag_version_suffix = (isset( $settings['widget_backward_compatibility'] ) && $settings['widget_backward_compatibility'] === 'yes') ? '' : '-v3';

		ob_start();
		?>
		<div style="min-height:20px;<?php echo $widget_wrapper; ?>">
			<quadpay-widget<?php echo $tag_version_suffix ?> amount="<?php echo $price; ?>" <?php echo $widget_customization; ?>></quadpay-widget<?php echo $tag_version_suffix ?>>
		</div>
		<?php

		$quadpay_widget = ob_get_clean();

		return $price_html . $quadpay_widget;

	}
	add_filter( 'woocommerce_get_price_html', 'quadpay_product_price_html_widget', 10, 2 );

	/**
	 * Showing the QuadPay widget on a variable product page
	 *
	 * @param $price_html
	 *
	 * @return string
	 */
	function quadpay_variable_price_html_widget( $price_html ) {

		if ( ! is_product() ) {
			return $price_html;
		}
		
		$settings = get_option( 'woocommerce_quadpay_settings' );

		if ( ! isset( $settings['enable_product_widget'] ) || $settings['enable_product_widget'] !== 'yes' || $settings['enabled'] !== 'yes' ) {
			return $price_html;
		}

		global $product;

		$widget_wrapper       = ! empty( $settings['product_page_widget_wrapper'] ) ? $settings['product_page_widget_wrapper'] : '';
		$widget_customization = ! empty( $settings['product_page_widget_customization'] ) ? $settings['product_page_widget_customization'] : '';

		$price = $product->get_price();
		$tag_version_suffix = (isset( $settings['widget_backward_compatibility'] ) && $settings['widget_backward_compatibility'] === 'yes') ? '' : '-v3';

		ob_start();
		?>
			<div style="min-height:20px;<?php echo $widget_wrapper; ?>">
                <quadpay-widget<?php echo $tag_version_suffix ?> amount="<?php echo $price; ?>" <?php echo $widget_customization; ?>></quadpay-widget<?php echo $tag_version_suffix ?>>
            </div>
		<?php

		$quadpay_widget = ob_get_clean();

		return $price_html . $quadpay_widget;

	}
	add_filter( 'woocommerce_variable_price_html', 'quadpay_variable_price_html_widget' );

	/**
	 * Showing the QuadPay widget on the Cart page
	**/
	function quadpay_info_cart_page_widget() {

		$ordertotal = WC()->cart->total;

		$settings = get_option('woocommerce_quadpay_settings');

		if ( ! isset( $settings['enable_cart_widget'] ) || $settings['enable_cart_widget'] !== 'yes' || $settings['enabled'] !== 'yes' ) {
			return;
		}

		$widget_wrapper       = ! empty ( $settings['cart_widget_wrapper'] ) ? $settings['cart_widget_wrapper'] : '';
		$widget_customization = ! empty ( $settings['cart_widget_customization'] ) ? $settings['cart_widget_customization'] : '';

		$tag_version_suffix = (isset( $settings['widget_backward_compatibility'] ) && $settings['widget_backward_compatibility'] === 'yes') ? '' : '-v3';

		ob_start();
		?>
			<div style="min-height:20px;<?php echo $widget_wrapper; ?>">
				<quadpay-widget<?php echo $tag_version_suffix ?> amount="<?php echo $ordertotal; ?>" <?php echo $widget_customization; ?>></quadpay-widget<?php echo $tag_version_suffix ?>>
			</div>
		<?php

		echo ob_get_clean();

	}
	add_action( 'woocommerce_proceed_to_checkout', 'quadpay_info_cart_page_widget', 15 );

	/**
	 * Call the cron task to check all pending orders status in the gateway
	 *
	 * @since 1.3.0
	 **/
	function quadpay_check_pending_orders_cron_job() {
		$gateway = WC_Gateway_QuadPay::instance();
		$gateway->check_pending_orders();
	}
	add_action( 'quadpay_thirty_minutes_cron_jobs', 'quadpay_check_pending_orders_cron_job' );

	/**
	 * Call the cron task to check all on-hold orders status in the gateway
	 *
	 * @since 1.3.0
	 **/
	function quadpay_check_on_hold_orders_cron_jobs() {
		$gateway = WC_Gateway_QuadPay::instance();
		$gateway->check_on_hold_orders();
	}
	add_action( 'quadpay_forty_five_minutes_cron_jobs', 'quadpay_check_on_hold_orders_cron_jobs' );

	/**
	 * Call the cron task to check the merchant payment limits on QuadPay
	 *
	 * @since 1.3.0
	 **/
	function quadpay_four_times_daily_cron_jobs() {
		$gateway = WC_Gateway_QuadPay::instance();
		$gateway->update_payment_limits();
	}
	add_action( 'quadpay_four_times_daily_cron_jobs', 'quadpay_four_times_daily_cron_jobs' );

	/**
	 * Custom WooCommerce Order Actions: Check payment status on QuadPay
	 *
	 * @param $order_actions
	 *
	 * @return mixed
	 *
	 * @since 1.3.0
	 */
	function quadpay_custom_woocommerce_order_action( $order_actions ) {

		global $post;
		global $theorder;

		// This is used by some callbacks attached to hooks such as woocommerce_order_actions which rely on the global to determine if actions should be displayed for certain orders.
		if ( ! is_object( $theorder ) ) {
			$theorder = wc_get_order( $post->ID );
		}

		$status         = $theorder->get_status();
		$payment_method = $theorder->get_payment_method();

		if ( 'quadpay' != $payment_method ) {
			return $order_actions;
		}

		if ( in_array( $status, array( 'processing', 'completed', 'refunded' ) ) ) {
			return $order_actions;
		}

		$order_actions['check_quadpay_status'] = 'Check payment status on QuadPay';

		return $order_actions;

	}
	add_filter( 'woocommerce_order_actions', 'quadpay_custom_woocommerce_order_action' );

	/**
	 * Process custom WooCommerce Order Actions: Check payment status on QuadPay
	 *
	 * @param $order
	 *
	 * @since 1.3.0
	 */
	function quadpay_woocommerce_check_order_payment_status( $order ) {

		$order_id       = $order->get_id();
		$status         = $order->get_status();
		$payment_method = $order->get_payment_method();

		if ( 'quadpay' != $payment_method ) {
			return;
		}

		if ( in_array( $status, array( 'processing', 'completed', 'refunded' ) ) ) {
			return;
		}

		$quadpay_order_token = get_post_meta( $order_id, '_quadpay_order_token', true );

		// Check if there's a stored order token. If not, it's not an QuadPay order.
		if ( ! $quadpay_order_token ) {
			return;
		}

		$gateway = WC_Gateway_QuadPay::instance();

		$response = wp_remote_get(
			$gateway->orderurl.'?token=' . $quadpay_order_token,
			array(
				'headers'=>array(
					'Authorization' => 'Bearer ' . $gateway->get_quadpay_authorization_code( 'quadpay_woocommerce_check_order_payment_status' ),
				)
			)
		);

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		$response_code = wp_remote_retrieve_response_code( $response );

		$response_message = wp_remote_retrieve_response_message( $response );

		// Check status of order
		if ( $response_code == 200 ) {

			// Check status of order
			if ( 'Approved' == $body->orderStatus ) {

				$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment approved. Zip Order ID: %s', 'woo_quadpay' ), $body->orderId ) );
				$order->payment_complete( $order_id );

			} elseif ( 'Created' == $body->orderStatus ) {

				$order->add_order_note( __( 'Checked payment status with Zip. Still pending approval.', 'woo_quadpay' ) );
				$order->update_status( 'pending' );

			} elseif ( 'Abandoned' == $body->orderStatus ) {

				$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment %s. Zip Order ID: %s', 'woo_quadpay' ), strtolower( $body->orderStatus ), $body->orderId ) );
				$order->update_status( 'cancelled' );

			} else {

				$order->add_order_note( sprintf( __( 'Checked payment status with Zip. Payment %s. Zip Order ID: %s', 'woo_quadpay' ), strtolower( $body->orderStatus ), $body->orderId ) );
				$order->update_status( 'failed' );

			}

		} else {

			$order->add_order_note( 'Can\'t check order payment status on Zip (response message: ' . $response_message . ').' );

		}

	}
	add_action( 'woocommerce_order_action_check_quadpay_status', 'quadpay_woocommerce_check_order_payment_status' );

	/**
	 * Enqueue scripts
	 *
	 * @since 1.0.0
	 **/
	function quadpay_wc_frontend_js() {
		wp_enqueue_script( 'quadpay_frontend', plugins_url( '/assets/js/frontend.js', __FILE__ ), array( 'jquery', 'quadpay_widget' ), '1.4.0', true );

		// Enqueue only on single product page, cart page or checkout page
		if ( is_singular( 'product' ) || is_cart() || is_checkout()) {

			$settings = get_option('woocommerce_quadpay_settings');

			if ( ! isset( $settings['enable_cart_widget'] ) || $settings['enable_cart_widget'] !== 'yes' || $settings['enabled'] !== 'yes' ) {
				return;
			}

			$widget_backward_compatibility = (isset( $settings['widget_backward_compatibility'] ) && $settings['widget_backward_compatibility'] === 'yes');

            wp_enqueue_script(
                    'quadpay_widget',
                    'https://cdn.quadpay.com/v1/quadpay.js' . ($widget_backward_compatibility ? '?tagname=quadpay-widget' : ''),
                    array(),
                    '1.4.0',
                    true
            );

		}

	}
	add_action( 'wp_enqueue_scripts', 'quadpay_wc_frontend_js' );

	if( !is_admin() ) {
		/**
		 * Add defer attribute
		 *
		 * @param $tag
		 * @param $handle
		 *
		 * @return string|string[]
		 *
		 * @since 1.3.8
		 */
		function add_defer_attribute( $tag, $handle ) {

			// if the unique handle/name of the registered script has 'defer' in it
			if ( strpos( $handle, 'defer' ) !== false ) {
				// return the tag with the defer attribute
				return str_replace( '<script ', '<script defer nomodule ', $tag );
			}
			// otherwise skip
			else {
				return $tag;
			}
		}
		add_filter('script_loader_tag', 'add_defer_attribute', 10, 2);
	}

}

/* WP-Cron activation and schedule setup */

/**
 * Schedule QuadPay WP-Cron job
 *
 * @since 1.0.0
 **/
function quadpay_wc_create_wpcronjob() {

	if ( ! wp_next_scheduled( 'quadpay_thirty_minutes_cron_jobs' ) ) {
		wp_schedule_event( time(),'thirtyminutes', 'quadpay_thirty_minutes_cron_jobs' );
	}

	if ( ! wp_next_scheduled( 'quadpay_forty_five_minutes_cron_jobs' ) ) {
		wp_schedule_event( time(),'fortyfiveminutes', 'quadpay_forty_five_minutes_cron_jobs' );
	}

	if ( ! wp_next_scheduled( 'quadpay_four_times_daily_cron_jobs' ) ) {
		wp_schedule_event( time(),'fourtimesdaily', 'quadpay_four_times_daily_cron_jobs' );
	}

}
add_action( 'wp', 'quadpay_wc_create_wpcronjob' );

/**
 * Delete QuadPay WP-Cron job
 *
 * @since 1.0.0
 **/
function quadpay_delete_wpcronjob() {

	wp_clear_scheduled_hook( 'quadpay_do_cron_jobs' );
	wp_clear_scheduled_hook( 'quadpay_thirty_minutes_cron_jobs' );
	wp_clear_scheduled_hook( 'quadpay_forty_five_minutes_cron_jobs' );
	wp_clear_scheduled_hook( 'quadpay_four_times_daily_cron_jobs' );

}
register_deactivation_hook( __FILE__, 'quadpay_delete_wpcronjob' );

/**
 * Add custom WP-Cron job scheduling intervals
 *
 * @param  array $schedules
 * @return array Array of schedules
 * @since 1.0.0
 **/
function quadpay_add_custom_cron_schedules( $schedules ) {

	$schedules['thirtyminutes'] = array(
		'interval' => 30 * 60,
		'display'  => __( 'Every 30 minutes', 'woo_quadpay' ),
	);

	$schedules['fortyfiveminutes'] = array(
		'interval' => 45 * 60,
		'display'  => __( 'Every 45 minutes', 'woo_quadpay' ),
	);

	$schedules['fourtimesdaily'] = array(
		'interval' => 6 * HOUR_IN_SECONDS,
		'display'  => __( '4 times a day', 'woo_quadpay' ),
	);

	return $schedules;
}
add_filter( 'cron_schedules', 'quadpay_add_custom_cron_schedules' );

/**
 * Add Settings link to the plugin entry in the plugins menu
 *
 * @since 1.3.0
 **/
function quadpay_woocommerce_plugin_action_links( $links ) {

	$settings_link = array(
		'settings' => sprintf( __( '<a href="%s" title="View WooCommerce Zip Gateway Settings">Settings</a>', 'woo_quadpay' ), admin_url( 'admin.php?page=wc-settings&tab=checkout&section=quadpay' ) ),
		'support'  => sprintf( __( '<a href="%s" title="Plugin Support" target="_blank">Support</a>', 'woo_quadpay' ), 'https://wordpress.org/support/plugin/quadpay-gateway-for-woocommerce/' ),
		'signup'  => sprintf( __( '<a href="%s" title="Create your Zip merchant account" target="_blank">Sign Up</a>', 'woo_quadpay' ), 'https://www.quadpay.com/signup-merchant/' )
	);

	return array_merge( $settings_link, $links );

}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'quadpay_woocommerce_plugin_action_links' );
